==> devil_viewer0.1.zip: devil_viewer0.1/source/devil_viewer.c <==
/*
 * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
 * Copyright (C) 1999-2003 Hiroyuki Yamamoto and the Sylpheed-Claws Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <gtk/gtk.h>
#include <gdk/gdkwin32.h>
#include <il/il.h>
#include <il/ilut.h>

#include "common/plugin.h"
#include "common/utils.h"
#include "mimeview.h"
#include "mainwindow.h"

typedef struct _devilViewer devilViewer;

struct _devilViewer
{
	MimeViewer	 mimeviewer;
	GtkWidget	*widget;	
	GtkWidget	*scrolledwin;
	GtkWidget	*image;
	GdkDrawable	*pixmap;
	gchar 		*filename;
	ILuint		 ImageName;
};

static MimeViewerFactory devil_viewer_factory;

static GtkWidget *devil_get_widget(MimeViewer *_viewer)
{
	devilViewer *viewer = (devilViewer *) _viewer;

	debug_print("devil_get_widget\n");

	return GTK_WIDGET(viewer->widget);
}

static gboolean scrolledwin_destroy_cb(GtkObject *object, gpointer data)
{
	devilViewer *viewer = (devilViewer *) data;
	debug_print("Destroyed devil scrolledwin %p\n", viewer->scrolledwin);
/*
	if (viewer->pixmap) {
		gdk_pixmap_unref(viewer->pixmap);
		viewer->image=NULL;
	}
*/
	if (viewer->image) {
		//gtk_widget_destroy(viewer->image);
		viewer->image = NULL;
	}
	if (viewer->scrolledwin) {
		//gtk_widget_destroy(viewer->scrolledwin);
		viewer->scrolledwin = NULL;
	}
	return FALSE;
}

static void get_resized_size(gint w, gint h, gint aw, gint ah,
			     gint *sw, gint *sh)
{
	gfloat wratio = 1.0;
	gfloat hratio = 1.0;
	gfloat ratio  = 1.0;

	if (w > aw)
		wratio = (gfloat)aw / (gfloat)w;
	if (h > ah)
		hratio = (gfloat)ah / (gfloat)h;

	ratio = (wratio > hratio) ? hratio : wratio;

	*sw = (gint)(w * ratio);
	*sh = (gint)(h * ratio);

	/* be paranoid */
	if (*sw <= 0 || *sh <= 0) {
		*sw = w;
		*sh = h;
	}
}

static void devil_show_mimepart(MimeViewer *_viewer, const gchar *infile, MimeInfo *partinfo)
{
	devilViewer *viewer = (devilViewer *) _viewer;
	HBITMAP hbmp;

	debug_print("devil_show_mimepart\n");

	if (viewer->filename != NULL) {
		unlink(viewer->filename);
		g_free(viewer->filename);
	}

	viewer->filename = procmime_get_tmp_file_name(partinfo);
	
	if (!(procmime_get_part(viewer->filename, infile, partinfo) < 0)) {
		int imwidth,imheight;
		int new_width,new_height;
		
		if (viewer->scrolledwin)
			gtk_widget_destroy(viewer->scrolledwin);

		ilGenImages(1, &(viewer->ImageName));
		ilBindImage(viewer->ImageName);
 
		if (!ilLoadImage(viewer->filename)) {
			ILenum Error;
			while ((Error = ilGetError()) != IL_NO_ERROR) {
			    debug_print("%d: %s/n", Error, iluErrorString(Error));
			}
			hbmp = 0;
			return;
		}

		imwidth  = ilGetInteger(IL_IMAGE_WIDTH);
		imheight = ilGetInteger(IL_IMAGE_HEIGHT);
		if (1) { /* XXX:tm resize */
			int avail_width = viewer->widget->parent->allocation.width;
			int avail_height = viewer->widget->parent->allocation.height;

			if (avail_width > 8) avail_width -= 8;
			if (avail_height > 8) avail_height -= 8;

			get_resized_size(imwidth, imheight,
				 avail_width, avail_height,
				 &new_width, &new_height);

			if (iluScale(new_width, new_height, 32))
				debug_print("scaled to %dx%d\n",new_width, new_height);
			else
				debug_print("ERROR:scaling to %dx%d\n",new_width, new_height);
		} else {
			new_width = imwidth;
			new_height = imheight;
		}

		hbmp = ilutConvertToHBitmap(NULL);
		SetBitmapDimensionEx(hbmp, new_width, new_height, NULL);
 
		viewer->pixmap = gdk_pixmap_foreign_new ((guint)hbmp);
		viewer->scrolledwin = gtk_scrolled_window_new(NULL, NULL);
		gtk_scrolled_window_set_policy(
					GTK_SCROLLED_WINDOW(viewer->scrolledwin),
					GTK_POLICY_AUTOMATIC,
					GTK_POLICY_AUTOMATIC);
		//gtk_widget_set_usize(scrolledwin, prefs_common.mainview_width, -1);
		gtk_widget_show(viewer->scrolledwin);

		viewer->image = gtk_pixmap_new(viewer->pixmap, NULL);
		gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(viewer->scrolledwin), viewer->image);
		gtk_widget_show(viewer->image);

		debug_print("Adding devil scrolledwin %p", viewer->scrolledwin);
		gtk_container_add(GTK_CONTAINER(viewer->widget),
				  viewer->scrolledwin);
		gtk_widget_realize(viewer->scrolledwin);
		gtk_widget_show(viewer->scrolledwin);
		gtk_signal_connect(GTK_OBJECT(viewer->scrolledwin), 
				   "destroy", 
				   GTK_SIGNAL_FUNC(scrolledwin_destroy_cb),
				   viewer);
	}
}

static void devil_clear_viewer(MimeViewer *_viewer)
{
	devilViewer *viewer = (devilViewer *) _viewer;
	debug_print("devil_clear_viewer\n");
	debug_print("Removing devil scrolledwin %p\n", viewer->scrolledwin);
	ilDeleteImages(1, &(viewer->ImageName));
/*
	if (viewer->pixmap) {
		gdk_pixmap_unref(viewer->pixmap);
		viewer->image=NULL;
	}
*/
	if (viewer->image) {
		gtk_widget_destroy(viewer->image);
		viewer->image=NULL;
	}
	if (viewer->scrolledwin) {
		gtk_widget_destroy(viewer->scrolledwin);
		viewer->scrolledwin=NULL;
	}
}

static void devil_destroy_viewer(MimeViewer *_viewer)
{
	devilViewer *viewer = (devilViewer *) _viewer;

	debug_print("devil_destroy_viewer\n");

	gtk_widget_unref(GTK_WIDGET(viewer->widget));
	unlink(viewer->filename);
	g_free(viewer->filename);
    	g_free(viewer);
}

static MimeViewer *devil_viewer_create()
{
	devilViewer *viewer;

	debug_print("devil_viewer_create\n");
	
	viewer = g_new0(devilViewer, 1);
	viewer->mimeviewer.factory = &devil_viewer_factory;

	viewer->mimeviewer.get_widget = devil_get_widget;
	viewer->mimeviewer.show_mimepart = devil_show_mimepart;
	viewer->mimeviewer.clear_viewer = devil_clear_viewer;
	viewer->mimeviewer.destroy_viewer = devil_destroy_viewer;	

	viewer->widget = gtk_event_box_new();
	gtk_widget_show(viewer->widget);
	gtk_widget_ref(viewer->widget);

	viewer->filename = NULL;

	return (MimeViewer *) viewer;
}

static MimeViewerFactory devil_viewer_factory =
{
	"image/*",
	0,
	
	devil_viewer_create,
};

gint plugin_init(gchar **error)
{
	ilInit();
	ilutInit();
	mimeview_register_viewer_factory(&devil_viewer_factory);
	return 0;	
}

void plugin_done()
{
	mimeview_unregister_viewer_factory(&devil_viewer_factory);
}

const gchar *plugin_name()
{
	return "devil Viewer";
}

const gchar *plugin_desc()
{
	return "Inline image view plugin, using DevIl library";
}

==> devil_viewer0.2.zip: devil_viewer0.2/source/devil_viewer.c <==
/*
 * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
 * Copyright (C) 1999-2003 Hiroyuki Yamamoto and the Sylpheed-Claws Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <gtk/gtk.h>
#include <gdk/gdkwin32.h>
#include <il/il.h>
#include <il/ilut.h>

#include "common/plugin.h"
#include "common/utils.h"
#include "mimeview.h"
#include "imageview.h"
#include "mainwindow.h"
#include "prefs_common.h"

typedef struct _devilViewer devilViewer;

struct _devilViewer
{
	MimeViewer	 mimeviewer;
	GtkWidget	*widget;	
	GtkWidget	*scrolledwin;
	GtkWidget	*image;
	GdkDrawable	*pixmap;
	gchar 		*filename;
	ILuint		 ImageName;
};

static MimeViewerFactory devil_viewer_factory;

static GtkWidget *devil_get_widget(MimeViewer *_viewer)
{
	devilViewer *viewer = (devilViewer *) _viewer;

	debug_print("devil_get_widget\n");

	return GTK_WIDGET(viewer->widget);
}

static gboolean scrolledwin_destroy_cb(GtkObject *object, gpointer data)
{
	devilViewer *viewer = (devilViewer *) data;
	debug_print("Destroyed devil scrolledwin %p\n", viewer->scrolledwin);
/*
	if (viewer->pixmap) {
		gdk_pixmap_unref(viewer->pixmap);
		viewer->image=NULL;
	}
*/
	if (viewer->image) {
		//gtk_widget_destroy(viewer->image);
		viewer->image = NULL;
	}
	if (viewer->scrolledwin) {
		//gtk_widget_destroy(viewer->scrolledwin);
		viewer->scrolledwin = NULL;
	}
	return FALSE;
}

static void devil_show_mimepart(MimeViewer *_viewer, const gchar *infile, MimeInfo *partinfo)
{
	devilViewer *viewer = (devilViewer *) _viewer;
	HBITMAP hbmp = NULL;
	PrefsCommon *prefs = prefs_common_get();

	if (!prefs->display_img)
		return;

	debug_print("devil_show_mimepart\n");

	if (viewer->filename != NULL) {
		unlink(viewer->filename);
		g_free(viewer->filename);
	}

	viewer->filename = procmime_get_tmp_file_name(partinfo);
	
	if (!(procmime_get_part(viewer->filename, infile, partinfo) < 0)) {
		int imwidth,imheight;
		int new_width,new_height;
		
		if (viewer->scrolledwin)
			gtk_widget_destroy(viewer->scrolledwin);

		ilGenImages(1, &(viewer->ImageName));
		ilBindImage(viewer->ImageName);
 
		if (!ilLoadImage(viewer->filename)) {
			ILenum Error;
			while ((Error = ilGetError()) != IL_NO_ERROR) {
			    debug_print("%d: %s/n", Error, iluErrorString(Error));
			}
			hbmp = 0;
			return;
		}

		imwidth  = ilGetInteger(IL_IMAGE_WIDTH);
		imheight = ilGetInteger(IL_IMAGE_HEIGHT);
		if (prefs->resize_image) {
			int avail_width = viewer->widget->parent->allocation.width;
			int avail_height = viewer->widget->parent->allocation.height;

			if (avail_width > 8) avail_width -= 8;
			if (avail_height > 8) avail_height -= 8;

			imageview_get_resized_size(imwidth, imheight,
				 avail_width, avail_height,
				 &new_width, &new_height);

			if (iluScale(new_width, new_height, 32))
				debug_print("scaled to %dx%d\n",new_width, new_height);
			else
				debug_print("ERROR:scaling to %dx%d\n",new_width, new_height);
		} else {
			new_width = imwidth;
			new_height = imheight;
		}

		hbmp = ilutConvertToHBitmap(NULL);
		SetBitmapDimensionEx(hbmp, new_width, new_height, NULL);
 
		viewer->pixmap = gdk_pixmap_foreign_new ((guint)hbmp);
		viewer->scrolledwin = gtk_scrolled_window_new(NULL, NULL);
		gtk_scrolled_window_set_policy(
					GTK_SCROLLED_WINDOW(viewer->scrolledwin),
					GTK_POLICY_AUTOMATIC,
					GTK_POLICY_AUTOMATIC);
		//gtk_widget_set_usize(scrolledwin, prefs_common.mainview_width, -1);
		gtk_widget_show(viewer->scrolledwin);

		viewer->image = gtk_pixmap_new(viewer->pixmap, NULL);
		gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(viewer->scrolledwin), viewer->image);
		gtk_widget_show(viewer->image);

		debug_print("Adding devil scrolledwin %p", viewer->scrolledwin);
		gtk_container_add(GTK_CONTAINER(viewer->widget),
				  viewer->scrolledwin);
		gtk_widget_realize(viewer->scrolledwin);
		gtk_widget_show(viewer->scrolledwin);
		gtk_signal_connect(GTK_OBJECT(viewer->scrolledwin), 
				   "destroy", 
				   GTK_SIGNAL_FUNC(scrolledwin_destroy_cb),
				   viewer);
	}
}

static void devil_clear_viewer(MimeViewer *_viewer)
{
	devilViewer *viewer = (devilViewer *) _viewer;
	debug_print("devil_clear_viewer\n");
	debug_print("Removing devil scrolledwin %p\n", viewer->scrolledwin);
	ilDeleteImages(1, &(viewer->ImageName));
/*
	if (viewer->pixmap) {
		gdk_pixmap_unref(viewer->pixmap);
		viewer->image=NULL;
	}
*/
	if (viewer->image) {
		gtk_widget_destroy(viewer->image);
		viewer->image=NULL;
	}
	if (viewer->scrolledwin) {
		gtk_widget_destroy(viewer->scrolledwin);
		viewer->scrolledwin=NULL;
	}
}

static void devil_destroy_viewer(MimeViewer *_viewer)
{
	devilViewer *viewer = (devilViewer *) _viewer;

	debug_print("devil_destroy_viewer\n");

	gtk_widget_unref(GTK_WIDGET(viewer->widget));
	unlink(viewer->filename);
	g_free(viewer->filename);
    	g_free(viewer);
}

static MimeViewer *devil_viewer_create()
{
	devilViewer *viewer;

	debug_print("devil_viewer_create\n");
	
	viewer = g_new0(devilViewer, 1);
	viewer->mimeviewer.factory = &devil_viewer_factory;

	viewer->mimeviewer.get_widget = devil_get_widget;
	viewer->mimeviewer.show_mimepart = devil_show_mimepart;
	viewer->mimeviewer.clear_viewer = devil_clear_viewer;
	viewer->mimeviewer.destroy_viewer = devil_destroy_viewer;	

	viewer->widget = gtk_event_box_new();
	gtk_widget_show(viewer->widget);
	gtk_widget_ref(viewer->widget);

	viewer->filename = NULL;

	return (MimeViewer *) viewer;
}

static MimeViewerFactory devil_viewer_factory =
{
	"image/*",
	0,
	
	devil_viewer_create,
};

gint plugin_init(gchar **error)
{
	ilInit();
	ilutInit();
	mimeview_register_viewer_factory(&devil_viewer_factory);
	return 0;	
}

void plugin_done()
{
	mimeview_unregister_viewer_factory(&devil_viewer_factory);
}

const gchar *plugin_name()
{
#ifdef _DEBUG
	return "devil Viewer 0.2 (debug)";
#else
	return "devil Viewer 0.2";
#endif
}

const gchar *plugin_desc()
{
	return "Inline image view plugin, using DevIl library";
}
